import logging

from pyramid.tweens import EXCVIEW
from pyramid.settings import aslist
from pyramid.util import DottedNameResolver
from pyramid.httpexceptions import WSGIHTTPException

resolver = DottedNameResolver(None)

import __builtin__

def as_globals_list(value):
    L = []
    value = aslist(value)
    for dottedname in value:
        if dottedname in __builtin__.__dict__:
            dottedname = '__builtin__.%s' % dottedname
        obj = resolver.resolve(dottedname)
        L.append(obj)
    return L

def exclog_tween_factory(handler, registry):

    get = registry.settings.get

    ignored = get('exclog.ignore', (WSGIHTTPException,))

    def exclog_tween(request, getLogger=logging.getLogger):
        # getLogger injected for testing purposes
        try:
            return handler(request)
        except ignored:
            raise
        except:
            logger = getLogger('exc_logger')
            logger.exception(request.url)
            raise

    return exclog_tween

def includeme(config):
    """
    Set up am implicit :term:`tween` to log exception information that is
    generated by your Pyramid application.  The logging data will be sent to
    the Python logger named ``exc_logger``.

    This tween configured to be placed 'below' the exception view tween.  It
    will log all exceptions (even those eventually caught by a Pyramid
    exception view) except 'http exceptions' (any exception that derives from
    ``pyramid.httpexceptions.WSGIHTTPException`` such as ``HTTPFound``).  You
    can instruct ``pyramid_exclog`` to ignore custom exception types by using
    the ``excview.ignore`` configuration setting.
    """
    get = config.registry.settings.get
    ignored = as_globals_list(get('exclog.ignore',
                                  'pyramid.httpexceptions.WSGIHTTPException'))
    config.registry.settings['exclog.ignore'] = tuple(ignored)
    config.add_tween('pyramid_exclog.exclog_tween_factory',
                     alias='exclog', under=EXCVIEW)



import unittest
from pyramid import testing

class Test_exclog_tween_factory(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.registry = self.config.registry

    def _callFUT(self):
        from pyramid_exclog import exclog_tween_factory
        return exclog_tween_factory(None, self.registry)

    def test_no_recipients(self):
        handler = self._callFUT()
        self.assertEqual(handler.__name__, 'exclog_tween')


class Test_exclog_tween(unittest.TestCase):
    def setUp(self):
        from pyramid.request import Request
        request = Request.blank('/')
        self.request = request
        self.config = testing.setUp(request=request)
        self.registry = self.config.registry
        self.registry.settings = {}
        request.registry = self.registry
        self.logger = DummyLogger()

    def getLogger(self, name):
        self.logger.name = name
        return self.logger

    def handler(self, request):
        raise NotImplementedError

    def _callFUT(self, handler=None, registry=None, request=None,
                 getLogger=None):
        from pyramid_exclog import exclog_tween_factory
        if handler is None:
            handler = self.handler
        if registry is None:
            registry = self.registry
        if request is None:
            request = self.request
        if getLogger is None:
            getLogger = self.getLogger
        tween = exclog_tween_factory(handler, registry)
        return tween(request, getLogger=getLogger)

    def test_ignored(self):
        self.registry.settings['exclog.ignore'] = (NotImplementedError,)
        self.assertRaises(NotImplementedError, self._callFUT)
        self.assertEqual(len(self.logger.exceptions), 0)

    def test_notignored(self):
        self.assertRaises(NotImplementedError, self._callFUT)
        self.assertEqual(len(self.logger.exceptions), 1)
        msg = self.logger.exceptions[0]
        self.assertEqual(msg, self.request.url)

class Test_includeme(unittest.TestCase):
    def _callFUT(self, config):
        from pyramid_exclog import includeme
        return includeme(config)

    def test_it(self):
        from pyramid.httpexceptions import WSGIHTTPException
        from pyramid.tweens import EXCVIEW
        config = DummyConfig()
        self._callFUT(config)
        self.assertEqual(config.tweens,
             [('pyramid_exclog.exclog_tween_factory', 'exclog', EXCVIEW, None)])
        self.assertEqual(config.registry.settings['exclog.ignore'],
                         (WSGIHTTPException,))

    def test_it_catchall(self):
        from pyramid.tweens import EXCVIEW
        config = DummyConfig()
        config.settings['exclog.catchall'] = 'true'
        self._callFUT(config)
        self.assertEqual(config.tweens,
                   [('pyramid_exclog.exclog_tween_factory', 'exclog', EXCVIEW,
                     None)])

    def test_it_withignored_builtin(self):
        config = DummyConfig()
        config.settings['exclog.ignore'] = 'NotImplementedError'
        self._callFUT(config)
        self.assertEqual(config.registry.settings['exclog.ignore'],
                         (NotImplementedError,))

    def test_it_withignored_nonbuiltin(self):
        config = DummyConfig()
        config.settings['exclog.ignore'] ='pyramid_exclog.tests.DummyException'
        self._callFUT(config)
        self.assertEqual(config.registry.settings['exclog.ignore'],
                         (DummyException,))

class DummyException(object):
    pass

class DummyLogger(object):
    def __init__(self):
        self.exceptions = []

    def exception(self, msg):
        self.exceptions.append(msg)

class DummyConfig(object):
    def __init__(self):
        self.tweens = []
        self.registry = self
        self.settings = {}

    def add_tween(self, factory, alias=None, under=None, over=None):
        self.tweens.append((factory, alias, under, over))

